@@ -1,7 +1,12 @@
 __author__ = 'Rens Holmer'
-__version__ = '0.0.15'
+__version__ = '0.0.16'
 
-from .tree import Tree, treeplot  # noqa
+from .tree import (  # noqa
+    Tree,
+    # TreeLayout,
+    calculate_tree_layout,
+    treeplot,
+)
 from .sequence import (  # noqa
     Alphabet,
     alphabets,
